package es.uv.garcosda.services;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import es.uv.garcosda.domain.Post;
import es.uv.garcosda.models.PostsRequestDTO;
import es.uv.garcosda.repositories.PostRepository;

@Service
@Transactional
public class BlogService {
	@Autowired
	PostRepository postRepository;

	public Page<Post> findPosts(PostsRequestDTO postsRequest) {
		postsRequest.setPageNo(postsRequest.getPageNo() < 0 ? 0 : postsRequest.getPageNo());
		postsRequest.setPageSize(postsRequest.getPageSize() < 1 ? 5 : postsRequest.getPageSize());
		
		Pageable pageable = PageRequest.of(postsRequest.getPageNo(), 
										   postsRequest.getPageSize(), 
										   Sort.by(Direction.DESC, "createdOn"));
		Page<Post> pageData = postRepository.findAll(pageable);

		return pageData;
	}

	public Optional<Post> findPostById(int postId) {
		return postRepository.findById(postId);
	}

	public Post createPost(Post post) {
		return postRepository.save(post);
	}

	public void deletePost(Integer postId) {
		postRepository.deleteById(postId);
	}
}
